==> fda.usc_0.8.tar.gz: fda.usc/R/fregre.plm.R <==
fregre.plm=function(formula,data,h=NULL,Ker=AKer.norm,metric=metric.lp,
type.CV = GCV.S,type.S=S.NW,par.CV=list(trim=0,draw=FALSE),...){
 C<-match.call()
 mf <- match.call(expand.dots = FALSE)
 m<-match(c("formula","data","h","Ker","metric","type.CV","type.S","par.CV"),names(mf),0L)
 tf <- terms.formula(formula)
 terms <- attr(tf, "term.labels")
 nt <- length(terms)
 if (attr(tf, "response") > 0) {
        response <- as.character(attr(tf, "variables")[2])
        pf <- rf <- paste(response, "~", sep = "")
    } else pf <- rf <- "~"
 vtab<-rownames(attr(tf,"factors"))
 vnf=intersect(terms,names(data$df))
 vnf2=intersect(vtab[-1],names(data$df)[-1])
 vfunc2=setdiff(terms,vnf)
 vint=setdiff(terms,vtab)
 vfunc=setdiff(vfunc2,vint)
 vnf=c(vnf2,vint)
 off<-attr(tf,"offset")
 kterms=1
 z=list()
 lenvnf=length(vnf)
 if (lenvnf>0) {
# cat(" Non functional variables: ",vnf,"\n")
 for ( i in 1:length(vnf)){
    if (kterms > 1)   pf <- paste(pf, "+", vnf[i], sep = "")
     else pf <- paste(pf, vnf[i], sep = "")
     kterms <- kterms + 1 }
 if   (attr(tf,"intercept")==0) {pf<- paste(pf,-1,sep="")}
 y=as.matrix(data[[1]][,response],ncol=1)
 n=nrow(y)
 if (length(vfunc)>0) {

  if (!is.fdata(data[[vfunc[1]]])) fdataobj=fdata(data[[vfunc[1]]])
  else fdataobj=data[[vfunc[1]]]
  x.fd<-fdataobj[["data"]]
  tt<-fdataobj[["argvals"]]
  rtt<-fdataobj[["rangeval"]]
#  if (class(data[[vfunc[1]]])[1]=="fd")   	 x.fd=t(data[[vfunc[1]]]$coefs)
#  else    	 x.fd=data[[vfunc[1]]]
#  if (is.data.frame(x.fd))  x.fd=as.matrix(x.fd)
#  cat(" ",class(data[[vfunc[1]]])[1]," object: ",vfunc[1],"\n")
   mdist=metric(x.fd,x.fd,...)
if (is.null(h)) {
if (m[7]==0){
     mdist2=mdist
#     diag(mdist2)=NA
    diag(mdist2)=Inf
     h0<-apply(mdist2,1,min,na.rm=TRUE)
     h.max=max(h0)
     h.med=median(h0)
     q.min=quantile(mdist2,probs=0.025,na.rm=TRUE)
     q.max=quantile(mdist2,probs=0.2,na.rm=TRUE)
     h.min=max(q.min,h.med)
     h.max=max(q.max,h.max)
     if (m[4]!=0) {
       i=1
       lenC=length(C)
       while (i<=lenC) {
        if (C[[i]]!="AKer.norm") {
                 h.max=min(q.max,h.max)
                 h.min=min(q.min,h.med)
                 i=lenC+1
                 }
        else i=i+1        }
       }
       else {h.max=min(q.max,h.max)
                h.min=min(q.min,h.med)}
     h=seq(h.min,h.max,len=51)
}
else if (C[[m[7]]]=="S.KNN") {
          h.min=2
          h.max=floor(quantile(1:n,probs=0.10,na.rm=TRUE,type=4))
          h=h.min:h.max     }
else {
     mdist2=mdist
     diag(mdist2)=NA
#    diag(mdist2)=Inf
     h0<-apply(mdist2,1,min,na.rm=TRUE)
     h.max=max(h0)
     h.med=median(h0)
     q.min=quantile(mdist2,probs=0.025,na.rm=TRUE)
     q.max=quantile(mdist2,probs=0.2,na.rm=TRUE)
     h.min=max(q.min,h.med)
     h.max=max(q.max,h.max)
     if (m[4]!=0) {
       i=1
       lenC=length(C)
       while (i<=lenC) {
        if (C[[i]]!="AKer.norm") {
                h.max=min(q.max,h.max)
                h.min=min(q.min,h.med)
                i=lenC+1
                 }
        else i=i+1        }
       }
       else {h.max=min(q.max,h.max)
                h.min=min(q.min,h.med)}
     h=seq(h.min,h.max,len=51)
}
 }
 lenh <- length(h)



 df=gcv<- array(NA, dim = c(lenh))
 yph <- array(NA, dim = c(nrow(y),lenh))
 H <- array(NA, dim = c(nrow(yph),nrow(y),lenh))
 I=diag(1,ncol=nrow(x.fd),nrow=nrow(x.fd))
 pb=txtProgressBar(min=0,max=lenh,style=3)
 XX=as.matrix(data[[1]][,vnf2])
 colnames(XX)=vnf2
 for (i in 1:lenh) {
    setTxtProgressBar(pb,i-0.5)
####  antes:
##    kmdist=Ker(mdist/h[i])
##    ww=kmdist/apply(kmdist, 1, sum)
####
   ww=type.S(mdist,h[i],Ker,cv=FALSE)
    wh=(I-ww)
    yh=wh%*%y
    xh=wh%*%XX
    betah=solve(t(xh)%*%xh)%*%t(xh)%*%yh
    mh=ww%*%(y-XX%*%betah)
    yph[,i]=XX%*%betah+mh
    e=yph[,i]-y
    c1=solve(t(xh)%*%xh)%*%t(xh)
    H[,,i]=wh%*%XX%*%c1%*%wh+ww
    df[i]=traza(H[,,i])
    y.pred3=H[,,i]%*%y
#    gcv[i] <- type.CV(y,H[,,i],trim=par.CV$trim,draw=par.CV$draw,...)
     par.CV$S<-H[,,i]
     par.CV$y<-y
     gcv[i]<- do.call(type.CV,par.CV)
     setTxtProgressBar(pb,i)
    }
close(pb)
  if (all(is.infinite(gcv))) print(" Warning: Invalid range for h")
  l = which.min(gcv)
  df=df[l]+ lenvnf
  h.opt <- h[l]
 	names(gcv)<-h
  yph=yph[,l]
  HH=H[,,l]
  e=y-yph
  sr2 = sum(e^2)/(n - df)
  ycen = y - mean(y)
  r2 = 1 - sum(e^2)/sum(ycen^2)
vcov2=sr2*solve(t(xh)%*%xh)
std.error=sqrt(diag(vcov2))
t.value=betah/std.error
p.value= 2 * pt(abs(t.value),df, lower.tail = FALSE)
result<-cbind(betah,std.error,t.value,p.value)
colnames(result) <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
if (lenh>1) {
  if (h.opt==min(h))  cat(" Warning: h.opt is the minimum value of bandwidths
   provided, range(h)=",range(h),"\n")
  else if (h.opt==max(h)) cat(" Warning: h.opt is the maximum value of bandwidths
   provided, range(h)=",range(h),"\n")}
z=list(coefficients=result,vcov=vcov2,r2=r2,residuals=e,sr2=sr2,formula=formula,h.opt=h.opt,h=h,fdataobjf=fdataobj,XX=XX,xh=xh,yh=yh,wh=wh,mdist=mdist,y=y,betah=betah,H=HH,data=data,call=C,fitted.values=yph,gcv=gcv,df=df,m=m,metric=metric,Ker=Ker,type.S=type.S)
class(z)="fregre.fd"
}
else {
  XX=data[[1]][,c(response,vnf2)]
  print("Warning: lm regession done, non functional data in the formula")
  if (!is.data.frame(XX)) XX=data.frame(XX)
      z=lm(formula=formula,data=XX,x=TRUE,y=TRUE,...)
      z$formula=formula
      z$data=data    }
}
else {
 print("Warning: fregre.np.cv done, only functional data in the formula")
 if (m[5]==0) {
  z=fregre.np.cv(data[[vfunc[1]]],matrix(data[[1]][,response],ncol=1),
  Ker=Ker,metric=metric,type.CV=type.CV,type.S=type.S,par.CV=par.CV,...)
 }
 else {
  a1<-match.fun(C[[m[5]]])
  for (i in 1:length(z$call)) {if (z$call[[i]]=="metric") {z$call[[i]]=metric}}
  z$metric=metric  }
 z$formula=formula
 z$data=data
 }
z
}

==> fda.usc_0.9.tar.gz: fda.usc/R/fregre.plm.R <==
fregre.plm=function(formula,data,h=NULL,Ker=AKer.norm,metric=metric.lp,
type.CV = GCV.S,type.S=S.NW,par.CV=list(trim=0,draw=FALSE),...){
 C<-match.call()
 mf <- match.call(expand.dots = FALSE)
 m<-match(c("formula","data","h","Ker","metric","type.CV","type.S","par.CV"),names(mf),0L)
 tf <- terms.formula(formula)
 terms <- attr(tf, "term.labels")
 nt <- length(terms)
 if (attr(tf, "response") > 0) {
        response <- as.character(attr(tf, "variables")[2])
        pf <- rf <- paste(response, "~", sep = "")
    } else pf <- rf <- "~"
 vtab<-rownames(attr(tf,"factors"))
 vnf=intersect(terms,names(data$df))
 vnf2=intersect(vtab[-1],names(data$df)[-1])
 vfunc2=setdiff(terms,vnf)
 vint=setdiff(terms,vtab)
 vfunc=setdiff(vfunc2,vint)
 vnf=c(vnf2,vint)
 off<-attr(tf,"offset")
 kterms=1
 z=list()
 lenvnf=length(vnf)
 ty<-deparse(substitute(type.S))
 ke<-deparse(substitute(Ker))
 if (lenvnf>0) {
# cat(" Non functional variables: ",vnf,"\n")
 for ( i in 1:length(vnf)){
    if (kterms > 1)   pf <- paste(pf, "+", vnf[i], sep = "")
     else pf <- paste(pf, vnf[i], sep = "")
     kterms <- kterms + 1 }
 if   (attr(tf,"intercept")==0) {pf<- paste(pf,-1,sep="")}
 y=as.matrix(data[[1]][,response],ncol=1)
 n=nrow(y)
 if (length(vfunc)>0) {

  if (!is.fdata(data[[vfunc[1]]])) fdataobj=fdata(data[[vfunc[1]]])
  else fdataobj=data[[vfunc[1]]]
  x.fd<-fdataobj[["data"]]
  tt<-fdataobj[["argvals"]]
  rtt<-fdataobj[["rangeval"]]
#  if (class(data[[vfunc[1]]])[1]=="fd")   	 x.fd=t(data[[vfunc[1]]]$coefs)
#  else    	 x.fd=data[[vfunc[1]]]
#  if (is.data.frame(x.fd))  x.fd=as.matrix(x.fd)
#  cat(" ",class(data[[vfunc[1]]])[1]," object: ",vfunc[1],"\n")
   mdist=metric(x.fd,x.fd,...)
   if (is.null(h))  h<-h.default(data[[vfunc[1]]],type.S=ty,metric=mdist,Ker=ke)
   lenh <- length(h)
   df=gcv<- array(NA, dim = c(lenh))
   yph <- array(NA, dim = c(nrow(y),lenh))
   H <- array(NA, dim = c(nrow(yph),nrow(y),lenh))
   I=diag(1,ncol=nrow(x.fd),nrow=nrow(x.fd))
   pb=txtProgressBar(min=0,max=lenh,style=3)
   XX=as.matrix(data[[1]][,vnf2])
   colnames(XX)=vnf2
   for (i in 1:lenh) {
        setTxtProgressBar(pb,i-0.5)
####  antes:
##    kmdist=Ker(mdist/h[i])
##    ww=kmdist/apply(kmdist, 1, sum)
####
    ww=type.S(mdist,h[i],Ker,cv=FALSE)
    wh=(I-ww)
    yh=wh%*%y
    xh=wh%*%XX
    betah=solve(t(xh)%*%xh)%*%t(xh)%*%yh
    mh=ww%*%(y-XX%*%betah)
    yph[,i]=XX%*%betah+mh
    e=yph[,i]-y
    c1=solve(t(xh)%*%xh)%*%t(xh)
    H[,,i]=wh%*%XX%*%c1%*%wh+ww
    df[i]=traza(H[,,i])
    y.pred3=H[,,i]%*%y
#    gcv[i] <- type.CV(y,H[,,i],trim=par.CV$trim,draw=par.CV$draw,...)
     par.CV$S<-H[,,i]
     par.CV$y<-y
     gcv[i]<- do.call(type.CV,par.CV)
     setTxtProgressBar(pb,i)
    }
close(pb)
  if (all(is.infinite(gcv))) print(" Warning: Invalid range for h")
  l = which.min(gcv)
  df=df[l]+ lenvnf
  h.opt <- h[l]
 	names(gcv)<-h
  yph=yph[,l]
  HH=H[,,l]
  e=y-yph
  sr2 = sum(e^2)/(n - df)
  ycen = y - mean(y)
  r2 = 1 - sum(e^2)/sum(ycen^2)
vcov2=sr2*solve(t(xh)%*%xh)
std.error=sqrt(diag(vcov2))
t.value=betah/std.error
p.value= 2 * pt(abs(t.value),df, lower.tail = FALSE)
result<-cbind(betah,std.error,t.value,p.value)
colnames(result) <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
if (lenh>1) {
  if (h.opt==min(h))  cat(" Warning: h.opt is the minimum value of bandwidths
   provided, range(h)=",range(h),"\n")
  else if (h.opt==max(h)) cat(" Warning: h.opt is the maximum value of bandwidths
   provided, range(h)=",range(h),"\n")}
z=list(coefficients=result,vcov=vcov2,r2=r2,residuals=e,sr2=sr2,formula=formula,h.opt=h.opt,h=h,fdataobjf=fdataobj,XX=XX,xh=xh,yh=yh,wh=wh,mdist=mdist,y=y,betah=betah,H=HH,data=data,call=C,fitted.values=yph,gcv=gcv,df=df,m=m,metric=metric,Ker=Ker,type.S=type.S)
class(z)="fregre.fd"
}
else {
  XX=data[[1]][,c(response,vnf2)]
  print("Warning: lm regession done, non functional data in the formula")
  if (!is.data.frame(XX)) XX=data.frame(XX)
      z=lm(formula=formula,data=XX,x=TRUE,y=TRUE,...)
      z$formula=formula
      z$data=data    }
}
else {
 print("Warning: fregre.np.cv done, only functional data in the formula")
 if (m[5]==0) {
  if (is.null(h)) h<-h.default(data[[vfunc[1]]],type.S=ty,Ker=ke)
  z=fregre.np.cv(data[[vfunc[1]]],matrix(data[[1]][,response],ncol=1),h=h,
  Ker=Ker,metric=metric,type.CV=type.CV,type.S=type.S,par.CV=par.CV,...)
 }
 else {
  a1<-match.fun(C[[m[5]]])
  for (i in 1:length(z$call)) {if (z$call[[i]]=="metric") {z$call[[i]]=metric}}
  z$metric=metric  }
 z$formula=formula
 z$data=data
 }
z
}
